""" Panoptes - AWS - Analysis

Responsible to the entire AWS analysis. Here the dynamic whitelist,
the logic behind unknown ingress rules and unused security groups are created.
"""

import boto3
import panoptes


CLOUD_PROVIDER = "aws"


def generate_unused_secgroup_entry(security_group: dict) -> dict:
    """
    Generates a dictionary from an unused security group to the analysis
    response
    """
    unused_group = {
        'GroupName': security_group['GroupName'],
        'GroupId': security_group['GroupId'],
        'Description': security_group['Description'],
        'VpcId': security_group.get('VpcId') or 'no-vpc',
    }
    return unused_group


def generate_unsafe_secgroup_entry(security_group: dict, unsafe_ingress_entries: list) -> dict:
    """
    Generates a dictionary from an unsafe security group, receiving all
    unsafe ingress entries related to this security group to the analysis
    response
    """
    unsafe_group = {
        "GroupName": security_group['GroupName'],
        "GroupId": security_group['GroupId'],
        "Description": security_group['Description'],
        "UnsafePorts": unsafe_ingress_entries,
    }
    return unsafe_group


def generate_unsafe_ingress_entry(ingress_entry: dict, unsafe_ip: str) -> dict:
    """
    Generates a dictionary from an unsafe ingress entry to the analysis
    response
    """
    unsafe_ingress = {
        "IpProtocol": ingress_entry["IpProtocol"],
        "CidrIp": unsafe_ip,
        "Status": "warning",
    }
    if "FromPort" in ingress_entry:
        unsafe_ingress["FromPort"] = ingress_entry["FromPort"]
    if "ToPort" in ingress_entry:
        unsafe_ingress["ToPort"] = ingress_entry["ToPort"]
    return unsafe_ingress


def analyze_unsafe_ingress(unsafe_ingress: dict) -> dict:
    """
    Receives an unsafe ingress and changes "Status" following some rules
    """
    def is_all_traffic(protocol: str) -> bool:
        ALL_TRAFFIC_PROTOCOL = "-1"
        return protocol == ALL_TRAFFIC_PROTOCOL
    def is_anywhere(cidr: str) -> bool:
        ANYWHERE_CIDR = "0.0.0.0/0"
        return cidr == ANYWHERE_CIDR

    if is_all_traffic(unsafe_ingress["IpProtocol"]):
        unsafe_ingress["Status"] = "alert"
    if is_anywhere(unsafe_ingress["CidrIp"]):
        unsafe_ingress["Status"] = "alert"

    return unsafe_ingress


def analyze_security_groups(session: boto3.session.Session, whitelist: list = []) -> dict:
    """
    The main analysis function

    Parameters:
        - aws_session:
            Type: boto3.Session
            Description: Client object from panoptes.aws.authentication.create_session() modules

        - whitelist:
            Type: list
            Description: List of whitelisted CIDR from optional input file

    DesiredReturn:
        {
            "Metadata": {
                "StartedAt": str[ISO 8601 Date],
                "FinishedAt": str[ISO 8601 Date],
                "CloudProvider": {
                    "Name": str,
                    "Auth": str,
                },
            },
            "SecurityGroups": {
                "UnusedGroups": [
                    {
                        "GroupName": str,
                        "GroupId": str,
                        "Description": str,
                        "VpcId": str,
                    }
                ],
                "UnsafeGroups": [
                    {
                        "GroupName": str,
                        "GroupId": str,
                        "Description": str,
                        "UnsafePorts": [
                            {
                                "FromPort": int,
                                "ToPort": int,
                                "IpProtocol": str,
                                "CidrIp": str,
                                "Status": str,
                            },
                        ]
                    },
                ],
            },
        }
    """
    response = {
        'SecurityGroups': {
            'UnusedGroups': [],
            'UnsafeGroups': [],
        },
        'Metadata': {
            'StartedAt': '',
            'FinishedAt': '',
            'CloudProvider': {
                'Name': '',
                'Auth': '',
            },
        },
    }
    response['Metadata']['StartedAt'] = panoptes.generic.helpers.get_current_time()

    whitelist += panoptes.aws.whitelist.list_all_safe_ips(session)
    all_security_groups = session.client('ec2').describe_security_groups()['SecurityGroups']
    all_attached_groups = panoptes.aws.attached.list_all_attached_secgroups(session)

    for security_group in all_security_groups:
        # Validating if group is unused
        if (
                security_group['GroupName'] not in all_attached_groups and
                security_group['GroupId'] not in all_attached_groups
        ):
            response['SecurityGroups']['UnusedGroups'].append(
                generate_unused_secgroup_entry(
                    security_group=security_group
                )
            )

        # Validating if group is unsafe
        unsafe_ingress_entries = []
        for ingress_entry in security_group['IpPermissions']:
            for allowed_ip in ingress_entry['IpRanges']:
                if allowed_ip['CidrIp'] not in whitelist:
                    unsafe_ingress = generate_unsafe_ingress_entry(
                        ingress_entry=ingress_entry,
                        unsafe_ip=allowed_ip['CidrIp'],
                    )
                    unsafe_ingress = analyze_unsafe_ingress(unsafe_ingress)
                    unsafe_ingress_entries.append(unsafe_ingress)

        if unsafe_ingress_entries:
            response['SecurityGroups']['UnsafeGroups'].append(
                generate_unsafe_secgroup_entry(
                    security_group=security_group,
                    unsafe_ingress_entries=unsafe_ingress_entries,
                )
            )

    response['Metadata']['FinishedAt'] = panoptes.generic.helpers.get_current_time()
    response['Metadata']['CloudProvider']['Name'] = CLOUD_PROVIDER
    response['Metadata']['CloudProvider']['Auth'] = (
        panoptes.aws.authentication.get_session_info(session)
    )
    return response


if __name__ == "__main__":
    pass



""" Panoptes - CLI - AWS
Responsible for organizing commands from Panoptes AWS CLI
"""

import click
import panoptes


AWS_AVAILABLE_OUTPUT_OPTIONS = [
    'human',
    'json',
    'yml',
]


@click.command(
    'analyze',
    help="Generate the analysis output"
)
@click.option(
    '-r', '--region',
    'region',
    required=True,
    help='AWS Region to list the security groups',
    metavar='<region_id>',
)
@click.option(
    '-p', '--profile',
    'profile',
    help='AWS CLI configured profile which will be used',
    metavar='<profile_name>',
)
@click.option(
    '-o', '--output',
    'output',
    default='human',
    help='Which kind of output you want the analysis',
    type=click.Choice(AWS_AVAILABLE_OUTPUT_OPTIONS),
)
@click.option(
    '--whitelist',
    'whitelist_path',
    help='Path to whitelist with declared safe IPs and CIDR',
    metavar='<path>',
)
def aws_analyze_command(region, profile, output, whitelist_path):
    """
    This function is called when the user types
    "panoptes aws analyze"
    """
    aws_output_options = {
        "human": panoptes.aws.output.print_human,
        "json": panoptes.generic.output.print_json,
        "yml": panoptes.generic.output.print_yml,
    }

    if whitelist_path:
        whitelist = panoptes.generic.helpers.parse_whitelist_file(
            whitelist_path=whitelist_path
        )
    else:
        whitelist = []

    session = panoptes.aws.authentication.create_session(
        region=region,
        profile=profile,
    )

    if session:
        analysis = panoptes.aws.analysis.analyze_security_groups(
            session=session,
            whitelist=whitelist,
        )
        print(aws_output_options.get(output)(analysis=analysis))


if __name__ == "__main__":
    pass



""" Panoptes - AWS
Responsible to import analysis functions from AWS.
"""

from panoptes.aws import analysis
from panoptes.aws import attached
from panoptes.aws import authentication
from panoptes.aws import exceptions
from panoptes.aws import output
from panoptes.aws import whitelist


if __name__ == "__main__":
    pass



#!/usr/bin/env python
import click
import pkg_resources
import panoptes.cli.aws
import panoptes.cli.gcp


@click.group()
def main():
    """Welcome to Panoptes - The multi cloud security group analyzer

    This project is stored on GitHub and open sourced under Apache 2.0

    To start using it, read the docs at:
    https://github.com/tioxy/panoptes
    """
    pass


@main.group(
    'aws',
    help='Amazon Web Services'
)
def aws_group():
    pass


@main.group(
    'gcp',
    help='Google Cloud Plataform'
)
def gcp_group():
    pass


@main.command(
    'version',
    help='Show Panoptes version'
)
def version_command():
    print(pkg_resources.get_distribution("panoptes").version)


"""
Adding commands to Click Groups
"""
aws_group.add_command(panoptes.cli.aws.aws_analyze_command)
gcp_group.add_command(panoptes.cli.gcp.gcp_analyze_command)


if __name__ == "__main__":
    main()
    exit()



""" Panoptes - AWS - Output

Functions to print specific AWS analysis output.
"""

import colorama
import jinja2
import panoptes


ALL_TRAFFIC_PROTOCOL = "-1"
COLOR_ALERT = colorama.Fore.LIGHTRED_EX
COLOR_WARNING = colorama.Fore.LIGHTYELLOW_EX
TEMPLATE = """{{ HEADER }}



Cloud provider  ->  {{ CLOUD_PROVIDER_NAME }}
Authentication  ->  {{ CLOUD_PROVIDER_AUTH }}
Started at      ->  {{ ANALYSIS_START_TIME }}
Finished at     ->  {{ ANALYSIS_END_TIME }}



{{ FIRST_SECTION }}
{% for secgroup in UNUSED_SECGROUPS %}{{ secgroup }}
{% endfor %}

{% for notification in UNUSED_SECGROUP_NOTIFICATIONS %}{{ notification }}
{% endfor %}


{{ SECOND_SECTION }}
{% for secgroup,rules in UNSAFE_SECGROUPS %}{{ secgroup }}
{{ rules }}
{% endfor %}
{% for notification in UNSAFE_RULES_NOTIFICATIONS %}{{ notification }}
{% endfor %}"""


def print_human(analysis):
    """
    Converts the AWS analysis dictionary into human readable output
    """
    def generate_ingress_message(protocol, range, cidr_ip, color):
        return (
            colorama.Style.RESET_ALL
            + colorama.Style.BRIGHT
            + color
            + "    "
            + protocol + "   " + range + "   " + cidr_ip
            + colorama.Style.RESET_ALL
        )

    def generate_security_group_message(security_group):
        return (
            colorama.Style.RESET_ALL
            + colorama.Style.BRIGHT
            + colorama.Fore.MAGENTA + security_group['GroupId']
            + "   "
            + colorama.Fore.WHITE + security_group['GroupName']
            + colorama.Style.RESET_ALL
        )

    human_output_template = jinja2.Template(TEMPLATE)
    unused_groups_list = analysis['SecurityGroups']['UnusedGroups']
    unsafe_groups_list = analysis['SecurityGroups']['UnsafeGroups']

    colorama.init()
    HEADER = panoptes.generic.output.generate_header_message(
        "PANOPTES Analysis"
    )

    FIRST_SECTION = panoptes.generic.output.generate_section_message(
        "01. UNUSED SECURITY GROUPS"
    )
    UNUSED_SECGROUP_NOTIFICATIONS = []
    if unused_groups_list:
        UNUSED_SECGROUPS = list(map(generate_security_group_message, unused_groups_list))
        UNUSED_SECGROUP_NOTIFICATIONS.append(
            panoptes.generic.output.generate_warning_message(
                f"{len(unused_groups_list)} security groups found not being used"
            )
        )
    else:
        UNUSED_SECGROUP_NOTIFICATIONS.append(
            panoptes.generic.output.generate_info_message(
                "All security groups are attached and being used"
            )
        )

    SECOND_SECTION = panoptes.generic.output.generate_section_message(
        "02. SECURITY GROUPS WITH UNSAFE INGRESS RULES"
    )
    UNSAFE_RULES_NOTIFICATIONS = []
    if unsafe_groups_list:
        alert_rules = 0
        warning_rules = 0
        UNSAFE_SECGROUPS = []
        for unsafe_group in unsafe_groups_list:
            secgroup = generate_security_group_message(unsafe_group)
            rules = ""

            for ingress in unsafe_group['UnsafePorts']:
                # Prettifying "protocol"
                protocol = ingress['IpProtocol'].upper()
                if protocol == ALL_TRAFFIC_PROTOCOL:
                    protocol = "All"

                # Prettifying "range"
                if any(k in ingress for k in ['FromPort', 'ToPort']):
                    if ingress['FromPort'] == ingress["ToPort"]:
                        range = (
                            f"{ingress['FromPort']}"
                        )
                    else:
                        range = (
                            f"{ingress['FromPort']}-{ingress['ToPort']}"
                        )
                else:
                    range = "All"

                if ingress['Status'] == "warning":
                    color = COLOR_WARNING
                    warning_rules += 1
                elif ingress['Status'] == "alert":
                    color = COLOR_ALERT
                    alert_rules += 1

                rules += (
                    generate_ingress_message(
                        protocol=protocol,
                        cidr_ip=ingress['CidrIp'],
                        range=range,
                        color=color,
                    )+'\n'
                )
            UNSAFE_SECGROUPS.append((secgroup, rules))
    
        if warning_rules:
            UNSAFE_RULES_NOTIFICATIONS.append(
                panoptes.generic.output.generate_warning_message(
                    f"{warning_rules} rules found with unknown IPs"
                )
            )
        if alert_rules:
            UNSAFE_RULES_NOTIFICATIONS.append(
                panoptes.generic.output.generate_alert_message(
                    f"{alert_rules} rules found with public IPs or all traffic enabled"
                )
            )
    else:
        UNSAFE_RULES_NOTIFICATIONS.append(
            panoptes.generic.output.generate_info_message(
                "All security groups have safe rules"
            )
        )

    start_time = panoptes.generic.helpers.generate_human_time(
        panoptes.generic.helpers.convert_string_datetime(analysis["Metadata"]["StartedAt"])
    )
    end_time = panoptes.generic.helpers.generate_human_time(
        panoptes.generic.helpers.convert_string_datetime(analysis["Metadata"]["FinishedAt"])
    )

    template_variables = {
        "HEADER": HEADER,
        "FIRST_SECTION": FIRST_SECTION,
        "UNUSED_SECGROUPS": UNUSED_SECGROUPS,
        "UNUSED_SECGROUP_NOTIFICATIONS": UNUSED_SECGROUP_NOTIFICATIONS,
        "SECOND_SECTION": SECOND_SECTION,
        "UNSAFE_SECGROUPS": UNSAFE_SECGROUPS,
        "UNSAFE_RULES_NOTIFICATIONS": UNSAFE_RULES_NOTIFICATIONS,
        "CLOUD_PROVIDER_NAME": analysis["Metadata"]["CloudProvider"]["Name"].upper(),
        "CLOUD_PROVIDER_AUTH": analysis["Metadata"]["CloudProvider"]["Auth"],
        "ANALYSIS_START_TIME": start_time,
        "ANALYSIS_END_TIME": end_time,
    }

    human_output = human_output_template.render(**template_variables)
    return human_output


if __name__ == "__main__":
    pass



""" Panoptes - Generic - Output
Responsible for generic outputs used through Panoptes module
"""

import json
import colorama
import yaml


def print_json(analysis: dict):
    """
    Converts the any analysis dictionary into prettified JSON output
    """
    return json.dumps(
        analysis,
        indent=4,
        sort_keys=True
    )


def print_yml(analysis: dict):
    """
    Converts the any analysis dictionary into YML output
    """
    return yaml.dump(
        analysis,
        allow_unicode=True,
        default_flow_style=False,
    )


def generate_alert_message(content: str):
    """
    Receives the ALERT message content and colorizes it
    """
    return (
        colorama.Style.RESET_ALL
        + colorama.Fore.LIGHTRED_EX
        + f"ALERT: {content}"
        + colorama.Style.RESET_ALL
    )


def generate_info_message(content: str):
    """
    Receives the INFO message content and colorizes it
    """
    return (
        colorama.Style.RESET_ALL
        + colorama.Fore.LIGHTCYAN_EX
        + f"INFO: {content}"
        + colorama.Style.RESET_ALL
    )


def generate_warning_message(content: str):
    """
    Receives the WARNING message content and colorizes it
    """
    return (
        colorama.Style.RESET_ALL
        + colorama.Fore.YELLOW
        + f"WARNING: {content}"
        + colorama.Style.RESET_ALL
    )


def generate_section_message(content: str):
    """
    Receives the SECTION message content and colorizes it
    """
    return (
        colorama.Style.RESET_ALL
        + colorama.Style.BRIGHT
        + colorama.Fore.LIGHTGREEN_EX
        + content
        + colorama.Style.RESET_ALL
    )


def generate_header_message(content: str, special_char: str = "=", special_len: int = 61):
    horizontal = special_len * special_char
    return(
        colorama.Style.RESET_ALL
        + colorama.Style.BRIGHT
        + colorama.Fore.LIGHTGREEN_EX
        + horizontal + "\n"
        + centralize_content_from_base_string(
            content=content,
            base_string=horizontal,
        ) + "\n"
        + horizontal
        + colorama.Style.RESET_ALL
    )


def centralize_content_from_base_string(content: str, base_string: str):
    def get_necessary_spaces(content: str, base_string: str):
        return ((len(base_string) - len(content)) // 2) * " "
    return (
        get_necessary_spaces(content, base_string)
        + content
        + get_necessary_spaces(content, base_string)
    )



""" Panoptes - AWS - Authentication

Using AWS best practices forcing the user to write Named Profiles instead of
direct IAM credentials through CLI.
"""

import boto3
import panoptes


def create_session(
        region: str,
        profile: str = None,
        session_token: str = None) -> boto3.session.Session:
    """
    Generates a Boto3 session from named profile and region inputs
    """
    session = boto3.Session(
        profile_name=profile,
        region_name=region,
        aws_session_token=session_token,
    )
    if not session.get_credentials():
        raise panoptes.aws.exceptions.PanoptesAWSCreateSessionError(
            "Panoptes could not authenticate to AWS. "
            "Check if your credentials exist and work."
        )
    return session


def get_session_info(session: boto3.session.Session) -> str:
    """
    Get ARN from the current session
    """
    return session.client('sts').get_caller_identity()['Arn']


def get_boto_clients(session: boto3.session.Session) -> dict:
    """
    Receives the session, and return all Panoptes used Boto3 Clients
    """
    return {
        'ec2': session.client('ec2'),
        'rds': session.client('rds'),
        'elb': session.client('elb'),
        'elbv2': session.client('elbv2'),
        'lambda': session.client('lambda'),
        'elasticache': session.client('elasticache'),
        'ecs': session.client('ecs'),
    }


if __name__ == "__main__":
    pass



""" Panoptes - AWS - Whitelist

Generates the dynamic whitelist of AWS Resources, considering them not harmful
and known resources
"""

import concurrent.futures
import boto3
import panoptes


def list_all_safe_ips(session: boto3.session.Session) -> list:
    """
    Function responsible for aggregating all methods and removing duplicates
    """
    all_safe_ips = []
    boto_clients = panoptes.aws.authentication.get_boto_clients(session)
    resources_to_whitelist = [
        (get_vpc_ranges, boto_clients['ec2']),
        (get_subnet_ranges, boto_clients['ec2']),
        (get_vpc_instance_ips, boto_clients['ec2']),
        (get_elastic_ips, boto_clients['ec2']),
    ]
    with concurrent.futures.ThreadPoolExecutor() as executor:
        running_workers = []
        for whitelist_function in resources_to_whitelist:
            running_workers.append(executor.submit(*whitelist_function))

        for future in concurrent.futures.as_completed(running_workers):
            all_safe_ips += future.result()
    return all_safe_ips


def get_vpc_ranges(ec2) -> list:
    """
    List VPCs CIDR ranges in the account
    """
    boto_vpcs = ec2.describe_vpcs()
    vpc_ranges = [
        vpc['CidrBlock'] for vpc in boto_vpcs['Vpcs']
    ]
    return vpc_ranges


def get_subnet_ranges(ec2) -> list:
    """
    List Subnets CIDR ranges in the account
    """
    boto_subnets = ec2.describe_subnets()
    subnet_ranges = [
        subnet['CidrBlock'] for subnet in boto_subnets['Subnets']
    ]
    return subnet_ranges


def get_vpc_instance_ips(ec2) -> list:
    """
    List Public and Private IPs from EC2 instances inside a VPC in the
    account
    """
    boto_instances = ec2.describe_instances()
    vpc_instances_ips = []
    for instance_obj in boto_instances['Reservations']:
        for instance in instance_obj['Instances']:
            for instance_net in instance['NetworkInterfaces']:
                if 'Association' in instance_net:
                    vpc_instances_ips.append(
                        instance_net['Association']['PublicIp'] + '/32'
                    )
                if 'PrivateIpAddress' in instance_net:
                    vpc_instances_ips.append(
                        instance_net['PrivateIpAddress'] + '/32'
                    )
                if 'PrivateIpAddresses' in instance_net:
                    for priv_ip in instance_net['PrivateIpAddresses']:
                        if 'Association' in priv_ip:
                            vpc_instances_ips.append(
                                priv_ip['Association']['PublicIp'] + '/32'
                            )
                        if 'PrivateIpAddress' in priv_ip:
                            vpc_instances_ips.append(
                                priv_ip['PrivateIpAddress'] + '/32'
                            )
    return vpc_instances_ips


def get_elastic_ips(ec2) -> list:
    """
    List all Elastic IPs reserved in the account
    """
    boto_elastic_ips = ec2.describe_addresses()
    elastic_ips = []
    for elastic_ip in boto_elastic_ips['Addresses']:
        if 'PrivateIpAddress' in elastic_ip:
            elastic_ips.append(
                elastic_ip['PrivateIpAddress'] + '/32'
            )
        elastic_ips.append(
            elastic_ip['PublicIp'] + '/32'
        )
    return elastic_ips


if __name__ == "__main__":
    pass



""" Panoptes - CLI - GCP
Responsible for organizing commands from Panoptes GCP CLI
"""

import click
import panoptes


GCP_AVAILABLE_OUTPUT_OPTIONS = [
    'human',
    'json',
    'yml',
]


@click.command(
    'analyze',
    help="Generate the analysis output"
)
def gcp_analyze_command():
    print("Nothing here")
    return None


if __name__ == "__main__":
    pass



""" Panoptes - AWS - Attached

Functions responsible for listing and grouping all attached security
groups within AWS resources.
"""

import concurrent.futures
import boto3
import panoptes


def list_all_attached_secgroups(session: boto3.session.Session) -> list:
    """
    Lists and groups all attached security groups within AWS resources
    """
    all_attached_groups = []
    boto_clients = panoptes.aws.authentication.get_boto_clients(session)

    services_with_security_groups = [
        (list_ec2_attached_secgroups, boto_clients['ec2']),
        (list_eni_attached_secgroups, boto_clients['ec2']),
        (list_rds_attached_secgroups, boto_clients['rds']),
        (list_elb_attached_secgroups, boto_clients['elb']),
        (list_elbv2_attached_secgroups, boto_clients['elbv2']),
        (list_lambda_attached_secgroups, boto_clients['lambda']),
        (list_elasticache_attached_secgroups, boto_clients['elasticache']),
        (list_ecs_attached_secgroups, boto_clients['ecs']),
    ]

    with concurrent.futures.ThreadPoolExecutor() as executor:
        running_workers = []
        for list_attached_function in services_with_security_groups:
            running_workers.append(executor.submit(*list_attached_function))

        for future in concurrent.futures.as_completed(running_workers):
            all_attached_groups += future.result()
    return all_attached_groups


def list_ec2_attached_secgroups(ec2) -> list:
    """
    List security groups attached to EC2 instances
    """
    ec2_attached_groups = []
    boto_ec2_instances = ec2.describe_instances()
    for instance_obj in boto_ec2_instances['Reservations']:
        for instance in instance_obj['Instances']:
            for security_group in instance['SecurityGroups']:
                ec2_attached_groups.append(
                    security_group['GroupId']
                )
    return ec2_attached_groups


def list_eni_attached_secgroups(ec2) -> list:
    """
    List security groups attached to Elastic Network Interfaces
    """
    network_interfaces = []
    next_token = None
    has_next = True
    while has_next:
        args = {'MaxResults': 1000}
        if next_token is not None:
            args['NextToken'] = next_token

        network_interfaces_result = ec2.describe_network_interfaces(**args)

        network_interfaces += network_interfaces_result['NetworkInterfaces']

        next_token = network_interfaces_result.get('NextToken')
        has_next = next_token is not None

    eni_attached_groups = [security_group['GroupId']
                           for network_interface in network_interfaces
                           for security_group in network_interface['Groups']]

    return eni_attached_groups


def list_rds_attached_secgroups(rds) -> list:
    """
    List security groups attached to RDS instances
    """
    rds_attached_groups = []
    boto_rds_instances = rds.describe_db_instances()
    for db_instance_obj in boto_rds_instances['DBInstances']:
        for security_group in db_instance_obj['VpcSecurityGroups']:
            rds_attached_groups.append(
                security_group['VpcSecurityGroupId']
            )
    return rds_attached_groups


def list_elb_attached_secgroups(elb) -> list:
    """
    List security groups attached to Elastic Load Balancers
    """
    elb_attached_groups = []
    boto_load_balancers = elb.describe_load_balancers()
    for elb_obj in boto_load_balancers['LoadBalancerDescriptions']:
        for security_group in elb_obj['SecurityGroups']:
            elb_attached_groups.append(
                security_group
            )
    return elb_attached_groups


def list_elbv2_attached_secgroups(elbv2) -> list:
    """
    List security groups attached to Elastic Load Balancers V2
    """
    elbv2_attached_groups = []
    boto_load_balancers = elbv2.describe_load_balancers()
    for elbv2_obj in boto_load_balancers['LoadBalancers']:
        if 'SecurityGroups' in elbv2_obj:
            for security_group in elbv2_obj['SecurityGroups']:
                elbv2_attached_groups.append(
                    security_group
                )
    return elbv2_attached_groups


def list_lambda_attached_secgroups(lambda_aws) -> list:
    """
    List security groups attached to Lambda functions
    """
    lambda_attached_groups = []
    boto_lambda = lambda_aws.list_functions()
    for lambda_obj in boto_lambda['Functions']:
        if 'VpcConfig' in lambda_obj:
            for security_group in (
                    lambda_obj['VpcConfig']['SecurityGroupIds']
            ):
                lambda_attached_groups.append(
                    security_group
                )
    return lambda_attached_groups


def list_elasticache_attached_secgroups(ecache) -> list:
    """
    List security groups attached to ElastiCache
    """
    elasticache_attached_groups = []
    boto_elasticache = ecache.describe_cache_clusters()
    for elasticache_obj in boto_elasticache['CacheClusters']:
        for security_group in elasticache_obj['CacheSecurityGroups']:
            elasticache_attached_groups.append(
                security_group['CacheSecurityGroupName']
            )
        if 'SecurityGroups' in elasticache_obj:
            for security_group in elasticache_obj['SecurityGroups']:
                elasticache_attached_groups.append(
                    security_group['SecurityGroupId']
                )
    try:
        boto_elasticache = ecache.describe_cache_security_groups()
        for elasticache_obj in boto_elasticache['CacheSecurityGroups']:
            for security_group in elasticache_obj['EC2SecurityGroups']:
                elasticache_attached_groups.append(
                    security_group['EC2SecurityGroupName']
                )
    except Exception as e:
        pass
    return elasticache_attached_groups


def list_ecs_attached_secgroups(ecs) -> list:
    """
    List security groups attached to ECS Services
    """
    ecs_attached_groups = []

    ecs_clusters = [
        ecs_clusters for ecs_clusters in ecs.list_clusters()['clusterArns']
    ]

    ecs_cluster_services = []
    for cluster in ecs_clusters:
        boto_services = ecs.list_services(cluster=cluster)
        if boto_services['serviceArns']:
            ecs_cluster_services.append(
                {
                    'ClusterName': cluster,
                    'Services': boto_services['serviceArns'],
                }
            )

    ECS_SERVICE_API_LIMIT = 10
    for cluster in ecs_cluster_services:
        for i in range(0, len(cluster['Services']), ECS_SERVICE_API_LIMIT):
            boto_ecs = ecs.describe_services(
                cluster=cluster['ClusterName'],
                services=cluster['Services'][i:i+ECS_SERVICE_API_LIMIT]
            )
            for ecs_obj in boto_ecs['services']:
                if 'networkConfiguration' in ecs_obj:
                    for security_group in (
                        ecs_obj['networkConfiguration']['awsvpcConfiguration']['securityGroups']
                    ):
                        ecs_attached_groups.append(
                            security_group
                        )
    return ecs_attached_groups



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['panoptes', 'panoptes.aws', 'panoptes.cli', 'panoptes.generic']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=2.11,<3.0',
 'PyYAML>=5.3,<6.0',
 'boto3>=1.14,<2.0',
 'click>=7.1,<8.0',
 'colorama>=0.4.3,<0.5.0']

entry_points = \
{'console_scripts': ['panoptesctl = panoptes.panoptesctl:main']}

setup_kwargs = {
    'name': 'panoptes',
    'version': '0.5.0',
    'description': 'The multi cloud security group analyzer',
    'long_description': '# PANOPTES\n\nPanoptes is an automatic security group analyzer focused on reducing the amount of toiling. Mainly focused on AWS, built to be extensible in the future and support the big cloud providers out there.\n\n<img src="https://s3.amazonaws.com/tioxy.github/panoptes/sample.gif" data-canonical-src="https://s3.amazonaws.com/tioxy.github/panoptes/sample.gif" width="600" height="900" />\n\n<br>\n\n## [Prerequisites](#prerequisites)\n-----\n\n#### For lazy users\nThere is a Docker image [*tioxy/panoptes*](https://hub.docker.com/r/tioxy/panoptes/) which uses the Dockerfile located in the root of the repository\n\n#### Python \n- Python >= 3.6\n- pip\n\n#### For AWS usage\n- [AWS CLI](https://docs.aws.amazon.com/cli/latest/userguide/installing.html) installed and configured with your IAM credentials (if you want more customization use [Named Profiles](https://docs.aws.amazon.com/cli/latest/userguide/cli-multiple-profiles.html))\n\n<br>\n\n## [Installation](#installation)\n-----\nGet the latest version of the package from your terminal with *pip*:\n```bash\npip install panoptes --upgrade\n```\n\n<br>\n\n## [Getting Started](#getting-started)\n\n-----\nIf you want to see the available options:\n```bash\npanoptesctl aws analyze --help\n```\n\nGenerate an AWS analysis with human readable output:\n```bash\npanoptesctl aws analyze --region <YOUR_REGION_CODE>\n```\n\nGenerate an AWS analysis with an YML output and a Named Profile from AWS CLI:\n```bash\npanoptesctl aws analyze --region <YOUR_REGION_CODE> --profile <YOUR_PROFILE> --output yml\n```\n*Check out [AWS Regions](https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-regions-availability-zones.html#concepts-available-regions) to see available region codes*\n\n\n<br>\n\n## [Whitelisting](#whitelisting)\n-----\n\n### Manually\nIf you wish to whitelist IPs, not letting the analysis mark them as **Unsafe**, you can define a sample file with the desired IPs and CIDRs, like the following one:\n\n<br>\n\n**your_whitelist.txt**\n```\n111.111.111.111/32\n123.123.123.123/32\n```\nAnd then run the analysis with the parameter *--whitelist*:\n```\npanoptesctl aws analyze --region <YOUR_REGION_CODE> --output yml --whitelist /PATH/TO/your_whitelist.txt\n```\n\n<br>\n\n### Automatically\nPanoptes generates automatically a list of IPs which it does not consider harmful from the desired cloud provider.\n\n<br>\n\n#### AWS autogenerated\n- VPC ranges\n- Subnet ranges\n- Private IPs from EC2 VPC Instances\n- Public IPs from EC2 VPC Instances\n- Elastic IPs\n\n<br>\n\n## [Limitations](#limitations)\n----\nThe Automatic AWS Whitelist feature can\'t whitelist *public* and *private* IP\'s from **EC2 Classic**, so make sure that those instances have an *Elastic IP* attached and their security groups are pointing to the new *Elastic IP* attached instead of the default EC2 Classic ones.\n',
    'author': 'Gabriel Tiossi',
    'author_email': 'gabrieltiossi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tioxy/panoptes',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



""" Panoptes - Main Module

Responsible only to import the cloud respective folders, which were created to
organize the project properly.
"""

from panoptes import aws
from panoptes import cli
from panoptes import generic
from panoptes import panoptesctl


if __name__ == "__main__":
    pass



""" Panoptes - Generic - Helpers

Just a collection of generic functions to help cloud provider analysis
"""

import datetime
import dateutil

def get_current_time() -> str:
    """
    Returns a datetime object with ISO 8601 format
    """
    return datetime.datetime.now().isoformat()


def generate_human_time(time: datetime.datetime) -> str:
    """
    Generates a human-readable string from a datetime object
    """
    return time.strftime("%b %d %Y, %H:%M:%S")


def convert_string_datetime(timestr: str) -> datetime.datetime:
    """
    Converts any string into a datetime object
    """
    return dateutil.parser.parse(timestr)


def parse_whitelist_file(whitelist_path: str) -> list:
    """
    Receives a whitelist_path containing 1 IP/CIDR per line and returns a list
    """
    with open(whitelist_path, 'r') as whitelist_file:
        whitelist = whitelist_file.read().splitlines()
    return whitelist



""" Panoptes - AWS - Exceptions

Collection of custom exceptions used to handle AWS errors
"""


class PanoptesAWSCreateSessionError(Exception):
    def __init__(self, message):
        super().__init__(message)
